% Replication code for Optima Debt Maturity Management
%(c) Bigio, Nuno & Passadore model
close all;  clear;
saveit=1; % Save figures if saveit==1

%%  PARAMETERS
% Code parameters
% Tolerance
parameters.tol_path   = 1e-3;  % tolerance parameter optimal path
parameters.tol        = 1e-4;  % tolerance parameter
parameters.max_iter   = 5000;  % maximum number of iterations
parameters.relax      = 0.1 ;  % relaxation coefficient
parameters.relax_path = 0.1;  % relaxation coefficient optimal path
parameters.plot_iter  = 0   ;  % one if you want to plot the iteration

% Numerical parameters
parameters.t_max     = 1000   ; % maximum number of years (time)
parameters.tau_max   = 30    ; % maximum number of years (maturity)
parameters.dt        = 1/12  ; % monthly steps

% Method to Solve the model
parameters.solve_method='iter'; % iter for iterative, solver for solver
parameters.flag = '0';


% Model parameters
% Time Period annual
% Preference Parameters
parameters.gamma        = 2;      % risk aversion

% adjustment cost
N_tau=parameters.tau_max/parameters.dt;
maturities     = [3 6 9 12 18 36 60 120 180 360];
parameters.lambda_bar_vec(1:N_tau,1)   = 1e15; % Computer precision. 
time_preallocate;

% Output 
y_ss                    = 1 ; 
parameters.y_ss         = y_ss ; % output at steady state
 
%Interest Rate
inflation               = 0.018;
parameters.delta        = 0.042 - inflation; 

% Price of Risk
parameters.U         =  @(c) (c.^(1-parameters.gamma)-1)/(1-parameters.gamma);
parameters.U_p_ratio =  @(c_a,c_b) (c_a/c_b).^(-parameters.gamma);
parameters.dU        =  @(c) (c.^(-parameters.gamma));

% Data
inflation_ss            = [0.018 0.018 0.018 0.018 0.018 0.019 0.020 0.021 0.022 0.023];                % Spanish inflation curve
curve_ss                = [0.027 0.033 0.038 0.041 0.045 0.049 0.050 0.051 0.052 0.052] - inflation_ss; % Spanish real yield curve 
issuances               = [0.91  1.16  2.31  3.32  1.33  1.75  2.01  2.62  0.77  0.63]*1e-2;            % Issuances over yearly GDP
lambda_bar              =  1e-2*[ 1.04 0.68 -2.02 -0.55 -5.99 10.49 15.06 17.27 39.45 53.70];


%% LIQUIDITY COSTS

rho   = 0.035;
r_bar = curve_ss;
delta = parameters.delta;

error_lambda =  @(rh) sum(abs(dt/2./issuances .* (1 - (exp(-rh*tau(maturities)) + (delta/rh).*(1 - exp(-rh*tau(maturities))))    ./ (exp(-r_bar.*tau(maturities)) + (delta./r_bar).*(1 - exp(-r_bar.*tau(maturities)))) )-lambda_bar ));
rho = fminsearch(error_lambda,rho);
lambda_bar_vec = dt/2./issuances .* (1 - (exp(-rho*tau(maturities)) + (delta/rho).*(1 - exp(-rho*tau(maturities))))    ./ (exp(-r_bar.*tau(maturities)) + (delta./r_bar).*(1 - exp(-r_bar.*tau(maturities)))) );
parameters.rho          = rho; % discount factor

%PLOT 
PLOT_liquidity_costs


%% TRANSITIONAL DYNAMICS

parameters.r_bar_ss     = curve_ss(8);
r_bar                   = parameters.r_bar_ss;
parameters.lambda_bar_vec(1:N_tau,1)   = 1e15; % Computer precision.
parameters.lambda_bar_vec(maturities) = lambda_bar(8)*2;

% % Output parameters 
parameters.y_0(1,1)       = y_ss  ;        % initial value, output after a shock
parameters.rho_y(1,1)     = (1-0.95)*4  ;        % persistence output after shock - 0.95 in quarterly data
parameters.rho_r_bar(1,1) = (1-0.95)*4;          % persistence (in years)
parameters.r_bar_0(1,1)   = parameters.r_bar_ss;  % initial value volatility of 0.048 quarterly


% Number of Shocks
N_r=   1;
N_y=   1;
parameters.N_r=N_r;
parameters.N_y=N_y;

% Update to a matrix of joint probabilities - discrete matrix of marginals
% - sum of row and columns must add up to 1
parameters.prob_mat=1; % [s*]

% No exogenous Issuances
parameters.Exo_Iota             = 0*parameters.lambda_bar_vec;

parameters.mode = '0'; % Perfect Foresight
% 
% No RSS
parameters.phi  = 0.00       ; % Poisson arrival rate
% 

% % Default parameters           - No default
parameters.defmodel ='logit'     ; % default model
parameters.muprobit =10;
parameters.penalty  = 1;        % Autarky penalty
% 


%% PLOT HRSS OBJECTS

% Method to Solve the model
parameters.solve_method='iter'; % iter for iterative, solver for solver
parameters.hrss_iter   = parameters.max_iter    ; % 
parameters.hrss_update = 0    ; % change to 0 for Galo's method, 1 for saki's method 
parameters.flag = '0';

parameters.mode = 'hr';
parameters.shock = 'y';
% Income shocks to the homothetic HRSS
parameters.hr.y_0(1:N_y,1)        = 0.99*y_ss       ;  % initial value, output after a shock
parameters.hr.rho_y(1:N_y,1)      = (1-0.95)*4  ;  % persistence output after shock - 0.95 in quarterly data
parameters.hr.r_bar_0(1:N_r,1)    = parameters.r_bar_ss       ;  % initial value volatility of 0.048 quarterly
parameters.hr.rho_r_bar(1:N_r,1)  = (1-0.95)*4  ;  % persistence (in years)

execute_codes
 if saveit==1
     save hrr_income;
 end

% % Rate shocks to the homothetic HRSS
parameters.mode = 'hr';
parameters.shock = 'r';
parameters.hr.y_0(1:N_y,1)        = y_ss       ;  % initial value, output after a shock
parameters.hr.rho_y(1:N_y,1)      = (1-0.95)*4  ;  % persistence output after shock - 0.95 in quarterly data
parameters.hr.r_bar_0(1:N_r,1)    = parameters.r_bar_ss   + 0.001    ;  % initial value volatility of 0.048 quarterly
parameters.hr.rho_r_bar(1:N_r,1)  = (1-0.95)*4  ;  % persistence (in years)

execute_codes
 if saveit==1
     save hrr_rates;
 end
 PLOT_IRFS



